package de.ozgcloud.formcycle;

import static lombok.AccessLevel.*;

import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.htmlcleaner.CleanerProperties;
import org.htmlcleaner.HtmlCleaner;

import lombok.NoArgsConstructor;

@NoArgsConstructor(access = PRIVATE)
public class HtmlUtils {

	private static final Pattern EMPTY_LINE_REGEX = Pattern.compile("(?m)^\\s*$(\\n|\\r\\n|\\r)");
	private static final HtmlCleaner htmlCleaner;

	static {
		var props = new CleanerProperties();
		props.setOmitComments(true);
		props.setOmitDoctypeDeclaration(true);
		props.setOmitXmlDeclaration(true);
		props.setOmitHtmlEnvelope(true);
		props.setOmitUnknownTags(true);
		props.setOmitDeprecatedTags(true);
		props.setPruneTags("script");
		htmlCleaner = new HtmlCleaner(props);
	}

	public static String removeHtmlTags(String withHtml) {
		if (StringUtils.isBlank(withHtml)) {
			return StringUtils.EMPTY;
		}
		var cleanedText = htmlCleaner.clean(withHtml).getText();
		cleanedText = removeEmptyLines(cleanedText);
		return isEndsWithNewLine(cleanedText)
				? cleanedText.subSequence(0, cleanedText.length() - 1).toString()
				: cleanedText.toString();
	}

	private static boolean isEndsWithNewLine(CharSequence text) {
		return text.length() > 0 && text.charAt(text.length() - 1) == '\n';
	}

	static CharSequence removeEmptyLines(CharSequence withEmptyLines) {
		if (StringUtils.isBlank(withEmptyLines)) {
			return StringUtils.EMPTY;
		}

		return EMPTY_LINE_REGEX.matcher(withEmptyLines).replaceAll(StringUtils.EMPTY);
	}

}
