package de.ozgcloud.formcycle.formdata;

import static de.ozgcloud.formcycle.formdata.ItemClass.*;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Deque;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

import de.xima.fc.form.common.items.XItem;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
class FormStructureParser {

	private final Map<String, XItem> formularElementsSorted;
	private final Deque<FormNode> containerNodes = new ArrayDeque<>();
	private final List<FormNode> nodes = new ArrayList<>();

	public List<FormNode> parse() {
		nodes.clear();
		for (Map.Entry<String, XItem> entry : formularElementsSorted.entrySet()) {
			var xItem = entry.getValue();
			var itemClass = ItemClass.fromString(xItem.getClassName());
			if (isInputNode(itemClass)) {
				addItem(xItem);
			} else if (isNestedContainer(itemClass)) {
				var formNode = buildNode(xItem);
				getNodes(xItem.getParentId()).add(formNode);
				containerNodes.push(formNode);
			}
		}
		return nodes;
	}

	void addItem(XItem xItem) {
		getNodes(xItem.getParentId()).add(buildNode(xItem));
	}

	FormNode buildNode(XItem xItem) {
		var formNode = FormNode.builder().itemId(xItem.getId()).name(xItem.getName());
		ItemClass.fromString(xItem.getClassName()).getTitleProperty().map(xItem::get).ifPresent(propertyValue -> formNode.title(propertyValue.getString()));
		return formNode.build();
	}

	List<FormNode> getNodes(String parentId) {
		return containerNodes.isEmpty()
				? nodes
				: getNextContainerNode(parentId).map(FormNode::getNestedElements).orElse(nodes);
	}

	Optional<FormNode> getNextContainerNode(String parentId) {
		var nextContainerNode = containerNodes.peekFirst();
		Objects.requireNonNull(nextContainerNode);
		while (nonParent(nextContainerNode, parentId)) {
			containerNodes.pollFirst();
			if (containerNodes.isEmpty()) {
				return Optional.empty();
			}
			nextContainerNode = containerNodes.peek();
		}
		return Optional.of(nextContainerNode);
	}

	private boolean nonParent(FormNode containerNode, String parentId) {
		return !containerNode.getItemId().equals(parentId);
	}
}
